"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __await = (this && this.__await) || function (v) { return this instanceof __await ? (this.v = v, this) : new __await(v); }
var __asyncGenerator = (this && this.__asyncGenerator) || function (thisArg, _arguments, generator) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var g = generator.apply(thisArg, _arguments || []), i, q = [];
    return i = {}, verb("next"), verb("throw"), verb("return", awaitReturn), i[Symbol.asyncIterator] = function () { return this; }, i;
    function awaitReturn(f) { return function (v) { return Promise.resolve(v).then(f, reject); }; }
    function verb(n, f) { if (g[n]) { i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; if (f) i[n] = f(i[n]); } }
    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }
    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
    function fulfill(value) { resume("next", value); }
    function reject(value) { resume("throw", value); }
    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mockListener = void 0;
const dispatcher_1 = require("@hatchet/protoc/dispatcher");
const sleep_1 = __importDefault(require("../../util/sleep"));
const nice_grpc_common_1 = require("nice-grpc-common");
const dispatcher_client_1 = require("./dispatcher-client");
const action_listener_1 = require("./action-listener");
const hatchet_client_test_1 = require("../hatchet-client/hatchet-client.test");
let dispatcher;
// Mock data for AssignedAction
const mockAssignedActions = [
    {
        tenantId: 'tenant1',
        jobId: 'job1',
        jobName: 'Job One',
        jobRunId: 'run1',
        stepId: 'step1',
        stepRunId: 'runStep1',
        actionId: 'action1',
        actionType: dispatcher_1.ActionType.START_STEP_RUN,
        actionPayload: 'payload1',
    },
    // ... Add more mock AssignedAction objects as needed
];
// Mock implementation of the listener
const mockListener = (fixture) => (function gen() {
    return __asyncGenerator(this, arguments, function* gen_1() {
        for (const action of fixture) {
            // Simulate asynchronous behavior
            yield __await((0, sleep_1.default)(100));
            if (action instanceof Error) {
                throw action;
            }
            yield yield __await(action);
        }
    });
})();
exports.mockListener = mockListener;
describe('ActionListener', () => {
    beforeEach(() => {
        dispatcher = new dispatcher_client_1.DispatcherClient({
            token: 'TOKEN',
            host_port: 'HOST_PORT',
            log_level: 'OFF',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, hatchet_client_test_1.mockChannel, hatchet_client_test_1.mockFactory);
    });
    it('should create a client', () => __awaiter(void 0, void 0, void 0, function* () {
        const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)(mockAssignedActions), 'WORKER_ID');
        expect(listener).toBeDefined();
        expect(listener.workerId).toEqual('WORKER_ID');
    }));
    describe('actions', () => {
        it('it should "yield" actions', () => __awaiter(void 0, void 0, void 0, function* () {
            var _a, e_1, _b, _c;
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)([...mockAssignedActions, new nice_grpc_common_1.ServerError(nice_grpc_common_1.Status.CANCELLED, 'CANCELLED')]), 'WORKER_ID');
            const actions = listener.actions();
            const res = [];
            try {
                for (var _d = true, actions_1 = __asyncValues(actions), actions_1_1; actions_1_1 = yield actions_1.next(), _a = actions_1_1.done, !_a; _d = true) {
                    _c = actions_1_1.value;
                    _d = false;
                    const action = _c;
                    res.push(action);
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = actions_1.return)) yield _b.call(actions_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            expect(res[0]).toEqual({
                tenantId: 'tenant1',
                jobId: 'job1',
                jobName: 'Job One',
                jobRunId: 'run1',
                stepId: 'step1',
                stepRunId: 'runStep1',
                actionId: 'action1',
                actionType: dispatcher_1.ActionType.START_STEP_RUN,
                actionPayload: 'payload1',
            });
        }));
        it('it should break on grpc CANCELLED', () => __awaiter(void 0, void 0, void 0, function* () {
            var _e, e_2, _f, _g;
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)([...mockAssignedActions, new nice_grpc_common_1.ServerError(nice_grpc_common_1.Status.CANCELLED, 'CANCELLED')]), 'WORKER_ID');
            const actions = listener.actions();
            const retrySpy = jest.spyOn(listener, 'retrySubscribe').mockResolvedValue(undefined);
            const res = [];
            try {
                for (var _h = true, actions_2 = __asyncValues(actions), actions_2_1; actions_2_1 = yield actions_2.next(), _e = actions_2_1.done, !_e; _h = true) {
                    _g = actions_2_1.value;
                    _h = false;
                    const action = _g;
                    res.push(action);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (!_h && !_e && (_f = actions_2.return)) yield _f.call(actions_2);
                }
                finally { if (e_2) throw e_2.error; }
            }
            expect(res.length).toEqual(1);
            expect(retrySpy).not.toHaveBeenCalled();
        }));
        it('it should break on unknown error', () => __awaiter(void 0, void 0, void 0, function* () {
            var _j, e_3, _k, _l;
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)([...mockAssignedActions, new Error('Simulated error')]), 'WORKER_ID');
            const actions = listener.actions();
            const retrySpy = jest.spyOn(listener, 'retrySubscribe').mockResolvedValue(undefined);
            const res = [];
            try {
                for (var _m = true, actions_3 = __asyncValues(actions), actions_3_1; actions_3_1 = yield actions_3.next(), _j = actions_3_1.done, !_j; _m = true) {
                    _l = actions_3_1.value;
                    _m = false;
                    const action = _l;
                    res.push(action);
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (!_m && !_j && (_k = actions_3.return)) yield _k.call(actions_3);
                }
                finally { if (e_3) throw e_3.error; }
            }
            expect(res.length).toEqual(1);
            expect(retrySpy).not.toHaveBeenCalled();
        }));
        it('it should attempt to re-establish connection on grpc UNAVAILABLE', () => __awaiter(void 0, void 0, void 0, function* () {
            var _o, e_4, _p, _q;
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)([...mockAssignedActions, new nice_grpc_common_1.ServerError(nice_grpc_common_1.Status.UNAVAILABLE, 'UNAVAILABLE')]), 'WORKER_ID');
            const retrySpy = jest.spyOn(listener, 'retrySubscribe').mockResolvedValue(undefined);
            const actions = listener.actions();
            const res = [];
            try {
                for (var _r = true, actions_4 = __asyncValues(actions), actions_4_1; actions_4_1 = yield actions_4.next(), _o = actions_4_1.done, !_o; _r = true) {
                    _q = actions_4_1.value;
                    _r = false;
                    const action = _q;
                    res.push(action);
                }
            }
            catch (e_4_1) { e_4 = { error: e_4_1 }; }
            finally {
                try {
                    if (!_r && !_o && (_p = actions_4.return)) yield _p.call(actions_4);
                }
                finally { if (e_4) throw e_4.error; }
            }
            expect(res.length).toEqual(1);
            expect(retrySpy).toHaveBeenCalled();
        }));
    });
    describe('retry_subscribe', () => {
        it('should exit after successful connection', () => __awaiter(void 0, void 0, void 0, function* () {
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)(mockAssignedActions), 'WORKER_ID');
            // Mock the listener to throw an error on the first call
            const listenSpy = jest
                .spyOn(listener.client, 'listen')
                .mockReturnValue((0, exports.mockListener)(mockAssignedActions));
            yield listener.retrySubscribe();
            expect(listenSpy).toHaveBeenCalledTimes(1);
        }));
        it('should retry until success', () => __awaiter(void 0, void 0, void 0, function* () {
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)(mockAssignedActions), 'WORKER_ID');
            // Mock the listener to throw an error on the first call
            const listenSpy = jest
                .spyOn(listener.client, 'listen')
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => (0, exports.mockListener)(mockAssignedActions));
            yield expect(() => __awaiter(void 0, void 0, void 0, function* () {
                yield listener.retrySubscribe();
            })).not.toThrow();
        }));
        it('should not throw an error if successful', () => __awaiter(void 0, void 0, void 0, function* () {
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)(mockAssignedActions), 'WORKER_ID');
            // Mock the listener to throw an error on the first call
            const listenSpy = jest
                .spyOn(listener.client, 'listen')
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => (0, exports.mockListener)(mockAssignedActions));
            yield listener.retrySubscribe();
            expect(listenSpy).toHaveBeenCalledTimes(2);
        }));
        it('should retry at most COUNT times and throw an error', () => __awaiter(void 0, void 0, void 0, function* () {
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)(mockAssignedActions), 'WORKER_ID');
            // Mock the listener to throw an error on the first call
            const listenSpy = jest
                .spyOn(listener.client, 'listen')
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => {
                throw new Error('Simulated error');
            })
                .mockImplementationOnce(() => (0, exports.mockListener)(mockAssignedActions));
            try {
                yield listener.retrySubscribe();
                expect(listenSpy).toHaveBeenCalledTimes(5);
            }
            catch (e) {
                expect(e.message).toEqual(`Could not subscribe to the worker after 5 retries`);
            }
        }));
    });
    describe('unregister', () => {
        it('should unsubscribe itself', () => __awaiter(void 0, void 0, void 0, function* () {
            const listener = new action_listener_1.ActionListener(dispatcher, (0, exports.mockListener)(mockAssignedActions), 'WORKER_ID');
            const unsubscribeSpy = jest.spyOn(listener.client, 'unsubscribe').mockResolvedValue({
                tenantId: 'TENANT_ID',
                workerId: 'WORKER_ID',
            });
            const res = yield listener.unregister();
            expect(unsubscribeSpy).toHaveBeenCalled();
            expect(res.workerId).toEqual('WORKER_ID');
        }));
    });
});
