"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dispatcher_1 = require("@hatchet/protoc/dispatcher");
const dispatcher_client_1 = require("./dispatcher-client");
const hatchet_client_test_1 = require("../hatchet-client/hatchet-client.test");
let client;
describe('DispatcherClient', () => {
    it('should create a client', () => {
        const x = new dispatcher_client_1.DispatcherClient({
            token: 'TOKEN',
            host_port: 'HOST_PORT',
            log_level: 'OFF',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, hatchet_client_test_1.mockChannel, hatchet_client_test_1.mockFactory);
        expect(x).toBeDefined();
    });
    beforeEach(() => {
        client = new dispatcher_client_1.DispatcherClient({
            token: 'TOKEN',
            host_port: 'HOST_PORT',
            log_level: 'OFF',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, hatchet_client_test_1.mockChannel, hatchet_client_test_1.mockFactory);
    });
    describe('get_action_listener', () => {
        it('should register the worker', () => __awaiter(void 0, void 0, void 0, function* () {
            const clientSpy = jest.spyOn(client.client, 'register').mockResolvedValue({
                workerId: 'WORKER_ID',
                tenantId: 'TENANT_ID',
                workerName: 'WORKER_NAME',
            });
            const listenerSpy = jest.spyOn(client.client, 'listen');
            const listener = yield client.get_action_listener({
                workerName: 'WORKER_NAME',
                services: ['SERVICE'],
                actions: ['ACTION'],
            });
            expect(clientSpy).toHaveBeenCalledWith({
                workerName: 'WORKER_NAME',
                services: ['SERVICE'],
                actions: ['ACTION'],
            });
            expect(listenerSpy).toHaveBeenCalledWith({
                workerId: 'WORKER_ID',
            });
            expect(listener).toBeDefined();
            expect(listener.workerId).toEqual('WORKER_ID');
        }));
    });
    describe('send_action_event', () => {
        it('should send action events', () => {
            const clientSpy = jest.spyOn(client.client, 'sendActionEvent').mockResolvedValue({
                tenantId: 'TENANT_ID',
                workerId: 'WORKER_ID',
            });
            client.send_action_event({
                workerId: 'WORKER_ID',
                actionId: 'ACTION_ID',
                eventType: dispatcher_1.ActionEventType.STEP_EVENT_TYPE_COMPLETED,
                eventPayload: '{"foo":"bar"}',
                eventTimestamp: new Date(),
                jobId: 'a',
                jobRunId: 'b',
                stepId: 'c',
                stepRunId: 'd',
            });
            expect(clientSpy).toHaveBeenCalledWith({
                workerId: 'WORKER_ID',
                actionId: 'ACTION_ID',
                eventType: dispatcher_1.ActionEventType.STEP_EVENT_TYPE_COMPLETED,
                eventPayload: '{"foo":"bar"}',
                jobId: 'a',
                jobRunId: 'b',
                stepId: 'c',
                stepRunId: 'd',
                eventTimestamp: expect.any(Object),
            });
        });
    });
});
