"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const event_client_1 = require("./event-client");
const hatchet_client_test_1 = require("../hatchet-client/hatchet-client.test");
let client;
describe('EventClient', () => {
    it('should create a client', () => {
        const x = new event_client_1.EventClient({
            token: 'TOKEN',
            host_port: 'HOST_PORT',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, hatchet_client_test_1.mockChannel, hatchet_client_test_1.mockFactory);
        expect(x).toBeDefined();
    });
    beforeEach(() => {
        client = new event_client_1.EventClient({
            token: 'TOKEN',
            host_port: 'HOST_PORT',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, hatchet_client_test_1.mockChannel, hatchet_client_test_1.mockFactory);
    });
    it('should push events', () => {
        const clientSpy = jest.spyOn(client.client, 'push').mockResolvedValue({
            tenantId: 'x',
            eventId: 'y',
            key: 'z',
            eventTimestamp: new Date(),
            payload: 'string',
        });
        client.push('type', { foo: 'bar' });
        expect(clientSpy).toHaveBeenCalledWith({
            key: 'type',
            payload: '{"foo":"bar"}',
            eventTimestamp: expect.any(Date),
        });
    });
    it('should throw an error when push fails', () => {
        const clientSpy = jest.spyOn(client.client, 'push');
        clientSpy.mockImplementation(() => {
            throw new Error('foo');
        });
        expect(() => {
            client.push('type', { foo: 'bar' });
        }).toThrow(new hatchet_error_1.default('foo'));
    });
});
