"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mockFactory = exports.mockChannel = void 0;
const nice_grpc_1 = require("nice-grpc");
const hatchet_client_1 = require("./hatchet-client");
exports.mockChannel = (0, nice_grpc_1.createChannel)('localhost:50051');
exports.mockFactory = (0, nice_grpc_1.createClientFactory)();
describe('Client', () => {
    beforeEach(() => {
        process.env.HATCHET_CLIENT_TOKEN = 'TOKEN_ENV';
    });
    it('should load from environment variables', () => {
        const hatchet = new hatchet_client_1.HatchetClient({
            token: 'TOKEN_ENV',
            host_port: 'HOST_PORT',
            log_level: 'OFF',
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, {
            credentials: nice_grpc_1.ChannelCredentials.createInsecure(),
        });
        expect(hatchet.config).toEqual({
            token: 'TOKEN_ENV',
            host_port: 'HOST_PORT',
            log_level: 'OFF',
            tls_config: {
                tls_strategy: 'tls',
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        });
    });
    it('should throw an error if the config param is invalid', () => {
        expect(() => new hatchet_client_1.HatchetClient({
            host_port: 'HOST_PORT',
            tls_config: {
                tls_strategy: 'tls',
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                // @ts-ignore
                server_name: undefined,
            },
        })).toThrow();
    });
    it('should favor config param over yaml over env vars ', () => {
        const hatchet = new hatchet_client_1.HatchetClient({
            tls_config: {
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        }, {
            config_path: './fixtures/.hatchet.yaml',
            credentials: nice_grpc_1.ChannelCredentials.createInsecure(),
        });
        expect(hatchet.config).toEqual({
            token: 'TOKEN_YAML',
            host_port: 'HOST_PORT_YAML',
            log_level: 'INFO',
            tls_config: {
                tls_strategy: 'tls',
                cert_file: 'TLS_CERT_FILE',
                key_file: 'TLS_KEY_FILE',
                ca_file: 'TLS_ROOT_CA_FILE',
                server_name: 'TLS_SERVER_NAME',
            },
        });
    });
    describe('with_host_port', () => {
        it('should set the host_port', () => {
            const hatchet = hatchet_client_1.HatchetClient.with_host_port('HOST', 1234, {
                token: 'TOKEN',
                tls_config: {
                    tls_strategy: 'tls',
                    cert_file: 'TLS_CERT_FILE',
                    key_file: 'TLS_KEY_FILE',
                    ca_file: 'TLS_ROOT_CA_FILE',
                    server_name: 'TLS_SERVER_NAME',
                },
            }, {
                credentials: nice_grpc_1.ChannelCredentials.createInsecure(),
            });
            expect(hatchet.config).toEqual({
                token: 'TOKEN',
                host_port: 'HOST:1234',
                log_level: 'INFO',
                tls_config: {
                    tls_strategy: 'tls',
                    cert_file: 'TLS_CERT_FILE',
                    key_file: 'TLS_KEY_FILE',
                    ca_file: 'TLS_ROOT_CA_FILE',
                    server_name: 'TLS_SERVER_NAME',
                },
            });
        });
    });
    describe('Worker', () => {
        let hatchet;
        beforeEach(() => {
            hatchet = new hatchet_client_1.HatchetClient({
                token: 'TOKEN',
                host_port: 'HOST_PORT',
                log_level: 'OFF',
                tls_config: {
                    cert_file: 'TLS_CERT_FILE',
                    key_file: 'TLS_KEY_FILE',
                    ca_file: 'TLS_ROOT_CA_FILE',
                    server_name: 'TLS_SERVER_NAME',
                },
            }, {
                credentials: nice_grpc_1.ChannelCredentials.createInsecure(),
            });
        });
        describe('run', () => {
            xit('should start a worker', () => {
                const worker = hatchet.run('workflow1');
                expect(worker).toBeDefined();
            });
        });
        describe('worker', () => {
            it('should start a worker', () => {
                const worker = hatchet.worker('workflow1');
                expect(worker).toBeDefined();
            });
        });
    });
});
