"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Worker = void 0;
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const dispatcher_1 = require("@hatchet/protoc/dispatcher");
const hatchet_promise_1 = __importDefault(require("../../util/hatchet-promise/hatchet-promise"));
const logger_1 = require("@hatchet/util/logger");
const sleep_1 = __importDefault(require("@hatchet/util/sleep"));
const step_1 = require("../../step");
class Worker {
    constructor(client, options) {
        this.serviceName = 'default';
        this.futures = {};
        this.client = client;
        this.name = options.name;
        this.action_registry = {};
        process.on('SIGTERM', () => this.exit_gracefully());
        process.on('SIGINT', () => this.exit_gracefully());
        this.killing = false;
        this.handle_kill = options.handleKill === undefined ? true : options.handleKill;
        this.logger = new logger_1.Logger(`Worker/${this.name}`, this.client.config.log_level);
    }
    register_workflow(workflow, options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.client.admin.put_workflow({
                    name: workflow.id,
                    description: workflow.description,
                    version: 'v0.55.0', // FIXME  workflow.version,
                    eventTriggers: workflow.on.event ? [workflow.on.event] : [],
                    cronTriggers: workflow.on.cron ? [workflow.on.cron] : [],
                    scheduledTriggers: [],
                    jobs: [
                        {
                            name: 'my-job', // FIXME variable names
                            timeout: '60s',
                            description: 'my-job',
                            steps: workflow.steps.map((step) => {
                                var _a;
                                return ({
                                    readableId: step.name,
                                    action: `${this.serviceName}:${step.name}`,
                                    timeout: '60s',
                                    inputs: '{}',
                                    parents: (_a = step.parents) !== null && _a !== void 0 ? _a : [],
                                });
                            }),
                        },
                    ],
                }, {
                    autoVersion: !(options === null || options === void 0 ? void 0 : options.autoVersion),
                });
            }
            catch (e) {
                throw new hatchet_error_1.default(`Could not register workflow: ${e.message}`);
            }
            this.action_registry = workflow.steps.reduce((acc, step) => {
                acc[`${this.serviceName}:${step.name}`] = step.run;
                return acc;
            }, {});
        });
    }
    handle_start_step_run(action) {
        const { actionId } = action;
        const context = new step_1.Context(action.actionPayload);
        const step = this.action_registry[actionId];
        if (!step) {
            this.logger.error(`Could not find step '${actionId}'`);
            return;
        }
        const run = () => __awaiter(this, void 0, void 0, function* () {
            return step(context);
        });
        const success = (result) => {
            this.logger.info(`Step run ${action.stepRunId} succeeded`);
            try {
                // Send the action event to the dispatcher
                const event = this.get_action_event(action, dispatcher_1.ActionEventType.STEP_EVENT_TYPE_COMPLETED, result);
                this.client.dispatcher.send_action_event(event);
                // delete the run from the futures
                delete this.futures[action.stepRunId];
            }
            catch (e) {
                this.logger.error(`Could not send action event: ${e.message}`);
            }
        };
        const failure = (error) => {
            this.logger.error(`Step run ${action.stepRunId} failed: ${error.message}`);
            try {
                // Send the action event to the dispatcher
                const event = this.get_action_event(action, dispatcher_1.ActionEventType.STEP_EVENT_TYPE_FAILED, error);
                this.client.dispatcher.send_action_event(event);
                // delete the run from the futures
                delete this.futures[action.stepRunId];
            }
            catch (e) {
                this.logger.error(`Could not send action event: ${e.message}`);
            }
        };
        const future = new hatchet_promise_1.default(run().then(success).catch(failure));
        this.futures[action.stepRunId] = future;
        try {
            // Send the action event to the dispatcher
            const event = this.get_action_event(action, dispatcher_1.ActionEventType.STEP_EVENT_TYPE_STARTED);
            this.client.dispatcher.send_action_event(event);
        }
        catch (e) {
            this.logger.error(`Could not send action event: ${e.message}`);
        }
    }
    get_action_event(action, eventType, payload = '') {
        return {
            workerId: this.name,
            jobId: action.jobId,
            jobRunId: action.jobRunId,
            stepId: action.stepId,
            stepRunId: action.stepRunId,
            actionId: action.actionId,
            eventTimestamp: new Date(),
            eventType,
            eventPayload: JSON.stringify(payload),
        };
    }
    handle_cancel_step_run(action) {
        const { stepRunId } = action;
        const future = this.futures[stepRunId];
        if (future) {
            future.cancel();
            delete this.futures[stepRunId];
        }
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.exit_gracefully();
        });
    }
    exit_gracefully() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.killing = true;
            this.logger.info('Starting to exit...');
            try {
                (_a = this.listener) === null || _a === void 0 ? void 0 : _a.unregister();
            }
            catch (e) {
                this.logger.error(`Could not unregister listener: ${e.message}`);
            }
            this.logger.info('Gracefully exiting hatchet worker, running tasks will attempt to finish...');
            // attempt to wait for futures to finish
            yield Promise.all(Object.values(this.futures).map(({ promise }) => promise));
            if (this.handle_kill) {
                this.logger.info('Exiting hatchet worker...');
                process.exit(0);
            }
        });
    }
    start() {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            let retries = 0;
            while (retries < 5) {
                try {
                    this.listener = yield this.client.dispatcher.get_action_listener({
                        workerName: this.name,
                        services: ['default'],
                        actions: Object.keys(this.action_registry),
                    });
                    const generator = this.listener.actions();
                    this.logger.info(`Worker ${this.name} listening for actions`);
                    try {
                        for (var _d = true, generator_1 = (e_1 = void 0, __asyncValues(generator)), generator_1_1; generator_1_1 = yield generator_1.next(), _a = generator_1_1.done, !_a; _d = true) {
                            _c = generator_1_1.value;
                            _d = false;
                            const action = _c;
                            this.logger.info(`Worker ${this.name} received action ${action.actionId}`);
                            if (action.actionType === dispatcher_1.ActionType.START_STEP_RUN) {
                                this.handle_start_step_run(action);
                            }
                            else if (action.actionType === dispatcher_1.ActionType.CANCEL_STEP_RUN) {
                                this.handle_cancel_step_run(action);
                            }
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (!_d && !_a && (_b = generator_1.return)) yield _b.call(generator_1);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    break;
                }
                catch (e) {
                    this.logger.error(`Could not start worker: ${e.message}`);
                    retries += 1;
                    const wait = 500;
                    this.logger.error(`Could not start worker, retrying in ${500} seconds`);
                    yield (0, sleep_1.default)(wait);
                }
            }
            if (this.killing)
                return;
            if (retries > 5) {
                throw new hatchet_error_1.default('Could not start worker after 5 retries');
            }
        });
    }
}
exports.Worker = Worker;
